''' List of generic utils functions
'''


def duration_string_to_seconds(string):
    ''' Converts a string representation of duration
        into total seconds in float.

        >>>  duration_string_to_seconds('00:03:16')
        196
    '''
    duration = string.split(':')
    return (
        int(duration[0]) * 60 * 60 +
        int(duration[1]) * 60 +
        int(duration[2])
    )


def duration_seconds_to_string(seconds):
    ''' Converts a total amount of seconds into a
        string representation of duration.

        >>>  duration_seconds_to_string(196)
        '00:03:16'
    '''
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    return f'{h}:{m:02}:{s:02}'



from __future__ import absolute_import

import time
import sys
import os
import select

from lyricfetch import get_lyrics, Song

from .players.sonos import SonosPlayer
from .utils import (
    duration_seconds_to_string,
    duration_string_to_seconds,
)


def retrieve_lyrics(track):
    # instantiate song
    song = Song.from_info(track['artist'], track['title'])
    # fetch lyrics
    get_lyrics(song)
    # read lyrics
    lyrics = song.lyrics.split('\n')
    return lyrics


def get_track_info(player):
    track = player.get_track_info()

    track['location'] = 0
    track['duration_secs'] = duration_string_to_seconds(track['duration'])
    track['position_secs'] = duration_string_to_seconds(track['position'])
    return track


def update_track_info(track):
    # calculate location
    percent = track['position_secs'] / track['duration_secs']
    track['location'] = int(round(percent * track['length'], 0))
    track['position'] = duration_seconds_to_string(track['position_secs'])
    return track


def display_lyrics(lyrics, track):
    '''
        Deals with formatting the lyrics unto the screen.

        Displays the lyrics on a 30 rows moving window.
    '''
    display_rows = 15
    separator = '-' * 60

    combine_lyrics = '\n'.join([
        ('@ ' if i == track['location'] else '| ') + row
        for i, row in
        enumerate(lyrics[
            max([0, min([
                track['length'] - (display_rows * 2),
                track['location'] - display_rows]
            )]):
            max([display_rows * 2, track['location'] + display_rows])
        ])
    ])
    print(
        '\n'.join([
              f'{separator}',
              f' {track["artist"]} - {track["title"]}',
              f'{separator}',
              '...' if track['location'] > display_rows else '',
              f'{combine_lyrics}',
              '...' if track['location'] < track['length'] - display_rows
              else '',
              f'{separator}',
              f' {track["position"]} / {track["duration"]}',
              f'{separator}',
              'Press ENTER to exit',
        ])
    )


def run():
    player = SonosPlayer()
    if not player.connected:
        return

    stop = False
    not_playing = False
    while True:
        track = get_track_info(player)

        if not track['title']:
            if not not_playing:
                print('Nothing playing')
                not_playing = True
            continue

        lyrics = retrieve_lyrics(track)
        track['length'] = len(lyrics)

        while True:
            os.system('clear')

            # display lyrics
            display_lyrics(lyrics, track)

            # refresh display in a second
            time.sleep(1)
            track['position_secs'] += 1
            track = update_track_info(track)

            # if ended exit
            if track['position_secs'] >= track['duration_secs']:
                break

            if sys.stdin in select.select([sys.stdin], [], [], 0)[0]:
                input()
                stop = True
                break

        if stop:
            break





class Player(object):
    def __init__(self):
        self.player = self.connect_player()
        self.connected = bool(self.player)

    def connect_player(self):
        raise NotImplementedError
        return 'player'

    def get_track_info(self):
        raise NotImplementedError
        return {
            'artist': '',
            'title': '',
            'duration': '',
            'position': '',
        }



from collections import OrderedDict

import soco

from .main import Player


class SonosPlayer(Player):
    def connect_player(self):
        # discover available sonos players
        self.players = OrderedDict({
            sonos.player_name: sonos
            for sonos in soco.discover()
        })

        print('Available Sonos Players:')
        print('\n'.join(
            '{}) {}'.format(i + 1, sonos)
            for i, sonos in enumerate(self.players.keys())
        ))

        # get the user choice
        sonos_id = self._user_input_choice()
        if not sonos_id:
            return

        # retrieve the IP address
        found_ip = self._retrieve_ip(sonos_id)
        print('Located sonos system //{}'.format(found_ip))

        return soco.SoCo(found_ip)

    def _user_input_choice(self):
        prompt = (
            'Please select the Sonos Player' +
            ' to connect to (or the first player in the group):\n'
        )
        # valid choices are either player names
        valid_choices = set(self.players.keys())
        # or the index in the printed list
        valid_choices |= set(range(1, len(self.players) + 1))

        tries = 0
        while tries < 3:
            sonos_id = input(prompt)

            if not sonos_id:
                return

            try:
                sonos_id = int(sonos_id)
            except ValueError:
                pass

            if sonos_id not in valid_choices:
                print('Invalid choice\n')
                tries += 1
                continue

            return sonos_id

    def _retrieve_ip(self, sonos_id):
        try:
            sonos_id = int(sonos_id)
        except ValueError:
            # sonos_id is either a string (player name)
            sonos = self.players[sonos_id]
        else:
            # or an integer (index in printed list)
            sonos = list(self.players.items())[int(sonos_id) - 1][1]
        return sonos.ip_address

    def get_track_info(self):
        return self.player.get_current_track_info()
